// opal -- online perceptron-like algorithm library
//  $Id: opal.cpp 533 2011-03-28 03:16:24Z ynaga $
// Copyright (c) 2009-2011 Naoki Yoshinaga <ynaga@tkl.iis.u-tokyo.ac.jp>
#include <cstring>
#include "opal_conf.h"
#include "pa.h"
#ifdef USE_TIMER
#include "timer.h"
#endif

int main (int argc, char** argv) {

  opal::option opt (argc, argv);

#ifdef USE_TIMER
  ny::Timer train_t ("train");
  ny::Timer test_t  ("test");
#endif
  opal::Model* m = opt.mode == opal::TEST ?
                   new opal::Model (opt.d, opt.N) :
                   new opal::Model (opt.d, opt.N, opt.algo, opt.ave, opt.s, opt.c, opt.M, opt.device);
  
  bool instant = std::strcmp (opt.model, "-") == 0;
  if (opt.mode == opal::TRAIN || opt.mode == opal::BOTH) {
#ifdef USE_TIMER
    train_t.startTimer ();
#endif
    if (std::strcmp (opt.model0, "-") != 0) m->load (opt.model0);
    m->train_from_file (opt.train, opt.iter, opt.debug == 2 ? opt.test : 0);
    if      (! instant) m->save (opt.model);
    else if (opt.ave)   m->average ();
#ifdef USE_TIMER
    train_t.stopTimer ();
#endif
  }
  if (opt.mode == opal::TEST || opt.mode == opal::BOTH) {
#ifdef USE_TIMER
    test_t.startTimer ();
#endif
    if (opt.mode == opal::BOTH && ! instant)
      { delete m; m = new opal::Model (opt.d, opt.N); }
    if (! instant) m->load (opt.model);
    m->test_on_file (opt.test, opt.debug);
#ifdef USE_TIMER
    test_t.stopTimer ();
#endif
  }
  delete m;
#ifdef USE_TIMER
  train_t.printElapsed ();
  test_t.printElapsed ();
#endif
  return 0;
}
